/*
 * GraphicsInserter.java
 *
 * The GraphicsInserter creates a graphics object on an empty text document 
 * by setting its position, width, height, and URL.
 *
 */
package org.openoffice.sdk.java.text;

import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.comp.helper.Bootstrap;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.text.TextContentAnchorType;
import com.sun.star.text.XText;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextDocument;
import com.sun.star.ucb.XFileIdentifierConverter;
import com.sun.star.ucb.XSimpleFileAccess;
import com.sun.star.uno.UnoRuntime;

import com.sun.star.uno.XComponentContext;

public class GraphicsInserter {    
    
    private XComponentContext m_xContext = null;
    private XMultiComponentFactory m_xMCF = null;
    private static String m_sGraphicFile;
    private static String m_sGraphicFileURL;
    
    public static void main(String args[]) {
        // arguments are passed-in using NetBeans IDE Project Properties
        // Properties - Run - Arguments
        // The sample images are included in this Project's folder
        if (args.length < 1) {
            System.out.println(
                    "usage: java -jar GraphicsInserter.jar \"<Graphic URL|path>\"");
            System.out.println("\ne.g.:");
            System.out.println(
                    "java -jar GraphicsInserter.jar \"C:\\TestGraphics.gif\"");
            System.exit(1);
        }
        
        m_sGraphicFile = args[0];
        // here check only the length of the arg
        // ... well, our file can only have one letter
        if (m_sGraphicFile.length()< 1){
            System.out.println("Please enter a valid graphic file path.");
            System.exit(1);
        }            
            
        GraphicsInserter graphicExample = new GraphicsInserter();
        
        try {            
            graphicExample.runDemo();
        } catch (java.lang.Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        } finally {
            System.exit(0);
        }
    }
    
    protected void runDemo() throws java.lang.Exception {
        // Check if the graphic file really exist
        // First convert the system path to an URL
        java.io.File sourceFile = new java.io.File(m_sGraphicFile);
        m_sGraphicFileURL = convertToURL("", sourceFile.getAbsolutePath());
        if (!checkFile(m_sGraphicFileURL)) {
            System.out.println("Impossible to locate the file " + m_sGraphicFileURL);
            System.exit(1);
        }

        XTextDocument xTextDoc = createTextDocument();

        // Querying for the interface XMultiServiceFactory on the XTextDocument
        XMultiServiceFactory xMSFDoc = (XMultiServiceFactory) 
                UnoRuntime.queryInterface(XMultiServiceFactory.class, xTextDoc);
        
        // Creating the service GraphicObject
        Object oGraphic = xMSFDoc.createInstance(
                "com.sun.star.text.TextGraphicObject");
        
        // Querying for the interface XTextContent on the GraphicObject
        XTextContent xTextContent = (XTextContent) UnoRuntime.queryInterface(
                XTextContent.class, oGraphic);
        
        // Getting the text
        XText xText = xTextDoc.getText();

        // Getting the cursor on the document
        XTextCursor xTextCursor = xText.createTextCursor();
        
        // Inserting the content
        xText.insertTextContent(xTextCursor, xTextContent, true);
        
        // Querying for the interface XPropertySet on the graphic object
        XPropertySet xPropSet = (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, oGraphic);
        
        // Setting the anchor type
        xPropSet.setPropertyValue("AnchorType", TextContentAnchorType.AT_PARAGRAPH);

        // Setting the graphic url
        xPropSet.setPropertyValue("GraphicURL", m_sGraphicFileURL);

        // Setting the horizontal position
        xPropSet.setPropertyValue("HoriOrientPosition", new Integer(5500));

        // Setting the vertical position
        xPropSet.setPropertyValue("VertOrientPosition", new Integer(4200));

        // Setting the width
        xPropSet.setPropertyValue("Width", new Integer(4400));

        // Setting the height
        xPropSet.setPropertyValue("Height", new Integer(4000));
    }
    
    
    private XMultiComponentFactory getMultiComponentFactory()
            throws java.lang.Exception {
        if (m_xContext == null && m_xMCF == null) {
            // get the remote office context. 
            // If necessary a new office process is started
            m_xContext = Bootstrap.bootstrap();
            System.out.println("Connected to a running office ...");
            m_xMCF = m_xContext.getServiceManager();
        }
        return m_xMCF;
    }
    
    private XComponent newDocComponent(String docType)
            throws java.lang.Exception {
        String loadUrl = "private:factory/" + docType;
        m_xMCF = this.getMultiComponentFactory();
        Object desktop = m_xMCF.createInstanceWithContext(
                "com.sun.star.frame.Desktop", m_xContext);
        XComponentLoader xComponentLoader = 
                (XComponentLoader) UnoRuntime.queryInterface(
                    XComponentLoader.class, desktop);
        PropertyValue[] loadProps = new PropertyValue[0];
        return xComponentLoader.loadComponentFromURL(loadUrl, "_blank",
                0, loadProps);
    }
    
    
    private XTextDocument createTextDocument() {
        XTextDocument aTextDocument = null;

        try {
            XComponent xComponent = newDocComponent("swriter");
            aTextDocument = (XTextDocument) UnoRuntime.queryInterface(
                    XTextDocument.class, xComponent);
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
        return aTextDocument;
    }
    
    
    private boolean checkFile( String aURL ){
        boolean bExists = false;
        try {
            XSimpleFileAccess xSFA = (XSimpleFileAccess) UnoRuntime.queryInterface(
                    XSimpleFileAccess.class, 
                    this.getMultiComponentFactory().createInstanceWithContext(
                        "com.sun.star.ucb.SimpleFileAccess", m_xContext));
            bExists = xSFA.exists(aURL) && !xSFA.isFolder(aURL);
        } catch (com.sun.star.ucb.CommandAbortedException ex){
            ex.printStackTrace();
        } catch (com.sun.star.uno.Exception ex){
            ex.printStackTrace();
        } catch (java.lang.Exception ex){
            ex.printStackTrace();
        }
        return bExists;
    }
    
    
    /**
     * Converts an URL into a system path using OOo API
     * @param sURLPath
     * @return
     */
    private String convertFromURL(String sURLPath) {
        String sSystemPath = null;
        try {
            m_xMCF = getMultiComponentFactory();
            XFileIdentifierConverter xFileConverter =
                    (XFileIdentifierConverter) UnoRuntime.queryInterface(
                        XFileIdentifierConverter.class,
                        m_xMCF.createInstanceWithContext(
                            "com.sun.star.ucb.FileContentProvider", m_xContext));
            sSystemPath = xFileConverter.getSystemPathFromFileURL(sURLPath);

        } catch (com.sun.star.uno.Exception e) {
            e.printStackTrace(System.err);
        } finally {
            return sSystemPath;
        }
    }
    
    /**
     * Converts a system path into an URL using OOo API
     * @param sBase
     * @param sSystemPath
     * @return
     */
    private String convertToURL(String sBase, String sSystemPath) {
        String sURL = null;
        try {
            m_xMCF = getMultiComponentFactory();
            XFileIdentifierConverter xFileConverter =
                    (XFileIdentifierConverter) UnoRuntime.queryInterface(
                        XFileIdentifierConverter.class,
                        m_xMCF.createInstanceWithContext(
                            "com.sun.star.ucb.FileContentProvider", m_xContext));
            sURL = xFileConverter.getFileURLFromSystemPath(
                    sBase, sSystemPath );
        } catch (com.sun.star.uno.Exception e) {
            e.printStackTrace();
        } finally {
            return sURL;
        }
    }
    
}
